#include <stdio.h>
#include <string.h>
#define	MACH_INIT_SLOTS		1
#include <mach.h>
#include <mach_error.h>
#include <mach/message.h>
#include <mach/bootstrap.h>
#include <mach/notify.h>
#include <mach/host_info.h>
#include <mach/host_reboot.h>
#include <mach/mach_host.h>
#include <mach/mach_interface.h>
#include <servers/service.h>
#include <servers/netname.h>

#include <servers/netname_server.h>

void printf_init(mach_port_t);
void panic_init(mach_port_t);
void panic(const char *, ...);

void random_run( mach_port_t port );
boolean_t timer_server( mach_msg_header_t*, mach_msg_header_t*);

char* gProgramName = "Counter_server";

mach_port_t notify_port;
mach_port_t netname_port;

mach_port_t root_ledger_wired;
mach_port_t root_ledger_paged;
mach_port_t security_port;
mach_port_t host_port;
mach_port_t device_port;

int
main(int argc, char *argv[]) {
    mach_port_t       ServerPort;
    kern_return_t kr = KERN_SUCCESS;

    gProgramName = argv[0];

	/* Do some standard initialization */
    kr = bootstrap_ports(bootstrap_port,
		&host_port,
		&device_port,
		&root_ledger_wired,
		&root_ledger_paged,
		&security_port);
    if (kr != KERN_SUCCESS) {
	printf("%s: bootstrap_ports: %s\n",
		gProgramName, mach_error_string(kr));
    	/*
    	* Let the bootstrap know we are finished, so it can load the next
    	* server at bootstrap time (if necessary).
    	*/
   	(void)bootstrap_completed(bootstrap_port, mach_task_self());
    }

	/* Initialize useful services.  We don't use panic, but c'est la vie */
    printf_init(device_port);
    panic_init(host_port);

   { /* Allocate ServerPort */
	kr = mach_port_allocate(mach_task_self(), 
			MACH_PORT_RIGHT_RECEIVE, &ServerPort);
	if ( kr != KERN_SUCCESS ) {
		mach_error("mach_port_allocate", kr);
    		/*
     		* Let the bootstrap know we are finished, so it can load the next
     		* server at bootstrap time (if necessary).
     		*/
   		(void)bootstrap_completed(bootstrap_port, mach_task_self());
		return 2;
	}
   }
   { /* Let everybody know about it via the netname server */
        kr = netname_check_in(name_server_port, "TimerServerPort",
                                   mach_task_self(),
                                   ServerPort);
	if ( kr != KERN_SUCCESS ) { 
		mach_error("netname_check_in", kr);
    		/*
     		* Let the bootstrap know we are finished, so it can load the next
     		* server at bootstrap time (if necessary).
     		*/
   		(void)bootstrap_completed(bootstrap_port, mach_task_self());
		return 2;
        }
   }

    /*
     * Let the bootstrap know we are finished, so it can load the next
     * server at bootstrap time (if necessary).
     */
   (void)bootstrap_completed(bootstrap_port, mach_task_self());

#if 1
	/* This is the nice & easy way to do things */
   kr = mach_msg_server( timer_server, DATA_SIZE * 3, ServerPort, MACH_MSG_OPTION_NONE );
   if ( kr != KERN_SUCCESS ) 
     mach_error("mach_msg_server", kr);
#else
	/* 
	 * The standard mach distribution for MkLinux doesn't like this.
	 * It leaks ((DATA_SIZE * 3) * 2) bytes of data each time through,
	 * when handling 'simpleroutine's and in error cases.
	 */
   while ( 1 ) { 
   	kr = mach_msg_server_once( timer_server, DATA_SIZE * 3, ServerPort, MACH_MSG_OPTION_NONE );
   	if ( kr != KERN_SUCCESS ) 
     		mach_error("mach_msg_server", kr);
   }
#endif

   printf("%s: Bye bye\n", gProgramName);
   return(1);
}
