

#include <mach.h>
#include <mach_error.h>
#include <mach/bootstrap.h>
#include <mach/clock_types.h>
#include <mach/clock.h>
#include <servers/netname.h>
#include <stdio.h>
#include <stdlib.h>
#include "timer.h"

#define ITERATION_COUNT (1024)

#define INLINE_ROUTINE 0
#define OUTOFLINE_ROUTINE 1
#define INLINE_SIMPLEROUTINE 0
#define OUTOFLINE_SIMPLEROUTINE 0

const char* gProgName = 0;

mach_port_t root_ledger_wired;
mach_port_t root_ledger_paged;
mach_port_t security_port;
mach_port_t host_port;
mach_port_t device_port;

int main ( int argc, char* argv[] ) { 
	kern_return_t kr;
	mach_port_t ServerPort;
	mach_port_t rt_clock;
	tvalspec_t cur_time;
 
	gProgName = argv[0];

	/* Standard initialization */
	kr = bootstrap_ports(bootstrap_port,
		&host_port,
		&device_port,
		&root_ledger_wired,
		&root_ledger_paged,
		&security_port);
	if (kr != KERN_SUCCESS) {
		mach_error("bootstrap_ports", kr);
		/* Let the bootstrapper know that it can continue */
		(void)bootstrap_completed(bootstrap_port, mach_task_self());
		return 2;
	}

	/* Let the bootstrapper know that it can continue */
	(void)bootstrap_completed(bootstrap_port, mach_task_self());

	/* Initialize some useful services */
	printf_init(device_port);
	panic_init(host_port);

	/* Find the advertised port of our server */
	kr = netname_look_up(name_server_port,
		"", "TimerServerPort",
		&ServerPort);
	if ( kr != KERN_SUCCESS ) {
		mach_error("netname_look_up(CounterServerPort)", kr);
		return 2;
	} 

	/* Get access to a timing clock */
	kr = host_get_clock_service ( host_port,
				REALTIME_CLOCK,
				&rt_clock );
	if ( kr != KERN_SUCCESS ) {
		mach_error("host_get_clock_service", kr);
		return 2;
	}

	/* Let's have fun and get the current time (we don't use it) */
	kr = clock_get_time(rt_clock, &cur_time );
	if ( kr != KERN_SUCCESS ) {
		mach_error("clock_get_time", kr);
		return 2;
	}


#if INLINE_ROUTINE
	{
		string_ptr buff;
		mach_msg_type_number_t buffCnt;
		tvalspec_t timea, timeb;
		int i;

		printf("Inline transfers\n");
		printf("-------------\n");

		/* Get the start time */
		kr = clock_get_time(rt_clock, &timea );
		if ( kr != KERN_SUCCESS ) {
			mach_error("clock_get_time", kr);
			return 2;
		}
		/* Do the messaging */
		for ( i = 0; i < ITERATION_COUNT; i++ ) {
			kr = get_secret(ServerPort, buff);
			if ( kr != KERN_SUCCESS )
				mach_error("get_random", kr);
			else { 
			}
		}
		/* Get the end time */
		kr = clock_get_time(rt_clock, &timeb );
		if ( kr != KERN_SUCCESS ) {
			mach_error("clock_get_time", kr);
			return 2;
		}
		printf("timea: { %u, %u }\ntimeb: { %u, %u }\ndiff: { %u, %u }\n", 
			timea.tv_sec, timea.tv_nsec,
			timeb.tv_sec, timeb.tv_nsec,
			timeb.tv_sec - timea.tv_sec, timeb.tv_nsec - timea.tv_nsec
			);
	}
#endif

#if OUTOFLINE_ROUTINE
	{
		tvalspec_t timea, timeb;
		string_ool_ptr password = 0;
		mach_msg_type_number_t passwordCnt = 0x000;
		int i;

		printf("OOL transfers\n");
		printf("-------------\n");

		kr = clock_get_time(rt_clock, &timea );
		if ( kr != KERN_SUCCESS ) {
			mach_error("clock_get_time", kr);
			return 2;
		}
		
		for ( i = 0; i < ITERATION_COUNT; i++ ) {
			kr = get_secret_ool(ServerPort,
					&password, &passwordCnt);
			if ( kr != KERN_SUCCESS )
				mach_error("get_secret_ool", kr);
			else { 
			}
			if ( password ) { 
				/* I think we should deallocate the memory coming via OOL data transfers */
				kr = vm_deallocate(mach_task_self(), 
						(vm_offset_t)password, passwordCnt);
				if ( kr != KERN_SUCCESS )
					mach_error("User:main:vm_deallocate", kr);
			}
		}

		kr = clock_get_time(rt_clock, &timeb );
		if ( kr != KERN_SUCCESS ) {
			mach_error("clock_get_time", kr);
			return 2;
		}
		printf("timea: { %u, %u }\ntimeb: { %u, %u }\ndiff: { %u, %u }\n", 
			timea.tv_sec, timea.tv_nsec,
			timeb.tv_sec, timeb.tv_nsec,
			timeb.tv_sec - timea.tv_sec, timeb.tv_nsec - timea.tv_nsec
			);
	}
#endif

#if INLINE_SIMPLEROUTINE
	{
		tvalspec_t timea, timeb;
		string_ptr password;
		int i;

		printf("Inline simpleroutines\n");
		printf("-------------\n");

		kr = clock_get_time(rt_clock, &timea );
		if ( kr != KERN_SUCCESS ) {
			mach_error("clock_get_time", kr);
			return 2;
		}

		for ( i = 0; i < ITERATION_COUNT; i++ ) {
			kr = tell_secret(ServerPort,
					password);
			if ( kr != KERN_SUCCESS )
				mach_error("tell_secret_ool", kr);
			else { 
			}
		}

		kr = clock_get_time(rt_clock, &timeb );
		if ( kr != KERN_SUCCESS ) {
			mach_error("clock_get_time", kr);
			return 2;
		}
		printf("timea: { %u, %u }\ntimeb: { %u, %u }\ndiff: { %u, %u }\n", 
			timea.tv_sec, timea.tv_nsec,
			timeb.tv_sec, timeb.tv_nsec,
			timeb.tv_sec - timea.tv_sec, timeb.tv_nsec - timea.tv_nsec
			);
	}
#endif

	return 0;
}

