
#ifdef __cplusplus
extern "C" { 
#endif
#include <stdio.h>
#include <stdlib.h>
#include <mach.h>
#include <mach_error.h>
#include <mig_errors.h>
#include <mach/message.h>
#include <mach/notify.h>
#include <servers/netname.h>
#ifdef __cplusplus
}
#endif

void random_run( mach_port_t port );

int main (int argc, char* argv[] ) {
   mach_port_t       ServerPort;
   kern_return_t kr;

   typedef struct supermsg {
     mach_msg_header_t header;
     int data;
   } supermsg_t;

   { // Allocate ServerPort
	kr = mach_port_allocate(mach_task_self(), 
			MACH_PORT_RIGHT_RECEIVE, &ServerPort);
	if ( kr != KERN_SUCCESS ) {
		mach_error("mach_port_allocate", kr);
		return 2;
	}
   }

   { // Create a send right on ServerPort
	kr = mach_port_insert_right( mach_task_self(),
				ServerPort,
				ServerPort,
				MACH_MSG_TYPE_MAKE_SEND);
	if ( kr != KERN_SUCCESS ) { 
		mach_error("mach_port_insert_right", kr);
		return 2;
	}
   }
		
   { // Send a message
     supermsg_t msg;
     msg.header.msgh_bits = MACH_MSGH_BITS(MACH_MSG_TYPE_MAKE_SEND, 0);
     msg.header.msgh_size = sizeof msg;
     msg.header.msgh_remote_port = ServerPort;
     msg.header.msgh_local_port = MACH_PORT_NULL;
     msg.header.msgh_reserved = 0;
     msg.header.msgh_id = 4321;
     {
	mach_msg_type_name_t dest_type = 
		MACH_MSGH_BITS_REMOTE(msg.header.msgh_bits);
	mach_msg_type_name_t reply_type = 
		MACH_MSGH_BITS_LOCAL(msg.header.msgh_bits);
	if ( !MACH_MSG_TYPE_PORT_ANY_SEND(dest_type))
		printf("No dest_type sends\n");
	if ((reply_type == 0) ? 
		(msg.header.msgh_local_port != MACH_PORT_NULL) :
		!MACH_MSG_TYPE_PORT_ANY_SEND(reply_type))
		printf("Second case failed\n");

	{
		mach_msg_return_t mr = mach_msg(
			(mach_msg_header_t*)&msg, 
			MACH_SEND_MSG,
			sizeof(msg), 0, MACH_PORT_NULL,
			MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
		if ( mr != MACH_MSG_SUCCESS ) {
			mach_error("mach_msg(SEND)", mr);
		} else printf("mach_msg(SEND) worked\n");
		printf("Sent: %u bytes\n", sizeof msg);
	}
     }
   }

   { // Examine port before receive
	mach_port_status_t info;
	mach_port_limits_t limits;
	mach_msg_type_number_t count = 0;
	count = MACH_PORT_RECEIVE_STATUS_COUNT;
	kr = mach_port_get_attributes(mach_task_self(),
			ServerPort,
			MACH_PORT_RECEIVE_STATUS,
			(mach_port_info_t)&info,
			&count);
	if ( kr != KERN_SUCCESS ) {
		mach_error("mach_port_get_attributes", kr);
	} else { 
		printf("\nmps_pset: %u\nmps_seqno: %u\nmps_mscount: %u\nmps_qlimit: %u\nmps_msgcount: %u\nmps_sorights: %u\nmps_srights: %u\nmps_pdrequest: %u\nmps_nsrequest: %u\nmps_flags: %u\n", info.mps_pset, info.mps_seqno, info.mps_mscount, info.mps_qlimit, info.mps_msgcount, info.mps_sorights, info.mps_srights, info.mps_pdrequest, info.mps_nsrequest, info.mps_flags);
	}
	count = MACH_PORT_LIMITS_INFO_COUNT;
	kr = mach_port_get_attributes(mach_task_self(),
			ServerPort,
			MACH_PORT_LIMITS_INFO,
			(mach_port_info_t)&limits,
			&count);
	if ( kr != KERN_SUCCESS ) {
		mach_error("mach_port_get_attributes", kr);
	} else { 
		printf( "	mpl_qlimit: %u\n", limits.mpl_qlimit );
	}
   }


   { // Receive message
     struct ReceiveMsg {
       supermsg_t message;
       mach_msg_trailer_t trailer;
     } msg;
     
     msg.message.header.msgh_bits =  0;
//MACH_MSGH_BITS(0, MACH_MSG_TYPE_MOVE_RECEIVE);
     msg.message.header.msgh_size = sizeof msg;
     msg.message.header.msgh_remote_port = MACH_PORT_NULL;
     msg.message.header.msgh_local_port = ServerPort;
     msg.message.header.msgh_reserved = 0;
     msg.message.header.msgh_id = 0;
     {
	mach_msg_return_t mr = mach_msg(
		(mach_msg_header_t*)(&msg), 
		MACH_RCV_MSG | MACH_RCV_TRAILER_ELEMENTS(0),
		0, msg.message.header.msgh_size, ServerPort,
		MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
	printf("Msgh_id: %u\n", msg.message.header.msgh_id);
	if ( mr != MACH_MSG_SUCCESS ) {
		mach_error("mach_msg_receive", mr);
		if ( mr == MACH_RCV_TOO_LARGE ) { 
			printf("Current_size: %u\nNeeded_size: %u\n", 
				sizeof msg, msg.message.header.msgh_size );
		} 
	} else {
		printf("mach_msg_receive worked\n");
		printf("msgh_id: %u\n", msg.message.header.msgh_id);
		printf("Trailer: { %u, %u }\n", 
			msg.trailer.msgh_trailer_type, 
			msg.trailer.msgh_trailer_size);
	}
     }
   }

   { // Examine port after receive
	mach_port_status_t info;
	mach_port_limits_t limits;
	mach_msg_type_number_t count = 0;
	count = MACH_PORT_RECEIVE_STATUS_COUNT;
	kr = mach_port_get_attributes(mach_task_self(),
			ServerPort,
			MACH_PORT_RECEIVE_STATUS,
			(mach_port_info_t)&info,
			&count);
	if ( kr != KERN_SUCCESS ) {
		mach_error("mach_port_get_attributes", kr);
	} else { 
		printf("\nmps_pset: %u\nmps_seqno: %u\nmps_mscount: %u\nmps_qlimit: %u\nmps_msgcount: %u\nmps_sorights: %u\nmps_srights: %u\nmps_pdrequest: %u\nmps_nsrequest: %u\nmps_flags: %u\n", info.mps_pset, info.mps_seqno, info.mps_mscount, info.mps_qlimit, info.mps_msgcount, info.mps_sorights, info.mps_srights, info.mps_pdrequest, info.mps_nsrequest, info.mps_flags);
	}
	count = MACH_PORT_LIMITS_INFO_COUNT;
	kr = mach_port_get_attributes(mach_task_self(),
			ServerPort,
			MACH_PORT_LIMITS_INFO,
			(mach_port_info_t)&limits,
			&count);
	if ( kr != KERN_SUCCESS ) {
		mach_error("mach_port_get_attributes", kr);
	} else { 
		printf("	mpl_qlimit: %u\n", limits.mpl_qlimit );
	}
   }

   { // Destroy ServerPort
   	kr = mach_port_destroy( mach_task_self(), ServerPort );
	if ( kr != KERN_SUCCESS ) {
		mach_error("mach_port_destroy", kr);
		return 2;
	}
   }

   printf("Bye\n");
   return 0;
}


