#include <stdio.h>
#include <string.h>
#define	MACH_INIT_SLOTS		1
#include <mach.h>
#include <mach_error.h>
#include <mach/message.h>
#include <mach/bootstrap.h>
#include <mach/notify.h>
#include <servers/service.h>
#include <servers/netname.h>

#include <servers/netname_server.h>

void printf_init(mach_port_t);
void panic_init(mach_port_t);
void panic(const char *, ...);

void random_run( mach_port_t port );
boolean_t random_server( mach_msg_header_t*, mach_msg_header_t*);

char* gProgramName = "Counter_server";

mach_port_t notify_port;
mach_port_t netname_port;

mach_port_t root_ledger_wired;
mach_port_t root_ledger_paged;
mach_port_t security_port;
mach_port_t host_port;
mach_port_t device_port;

int
main(int argc, char *argv[]) {
    mach_port_t       ServerPort;
    kern_return_t kr = KERN_SUCCESS;

    gProgramName = argv[0];

    kr = bootstrap_ports(bootstrap_port,
		&host_port,
		&device_port,
		&root_ledger_wired,
		&root_ledger_paged,
		&security_port);
    if (kr != KERN_SUCCESS) {
	panic("%s: bootstrap_ports: %s\n",
		gProgramName, mach_error_string(kr));
    }

    printf_init(device_port);
    panic_init(host_port);

    printf("%s: started\n", gProgramName);

   { // Allocate ServerPort
	kr = mach_port_allocate(mach_task_self(), 
			MACH_PORT_RIGHT_RECEIVE, &ServerPort);
	if ( kr != KERN_SUCCESS ) {
		mach_error("mach_port_allocate", kr);
		return 2;
	}
   }
   { // Let everybody know about it
        kr = netname_check_in(name_server_port, "CounterServerPort",
                                   mach_task_self(),
                                   ServerPort);
	if ( kr != KERN_SUCCESS ) { 
		mach_error("netname_check_in", kr);
		return 2;
        }
   }

    /*
     * Let the bootstrap know we are finished.
     */
   (void)bootstrap_completed(bootstrap_port, mach_task_self());

   kr = mach_msg_server( random_server, 
			(1024*1024),
			ServerPort,
			MACH_MSG_OPTION_NONE );
   mach_error("mach_msg_server", kr);
   printf("%s: Bye bye\n", gProgramName);
   return(1);
}

